/*
 * ndspy.h - API between renderer and DSO display drivers
 *
 * RenderDotC (R) is:
 *      Copyright (c) 1996-2001 by Dot C Software, Inc.
 *      All Rights Reserved.
 * RenderDotC (R) is a registered trademark of Dot C Software, Inc.
 */

#ifndef NDSPY_H
#define NDSPY_H

#include "ridefs.h"		/* For RtPointer */

#undef RI_IMPORT
#undef RI_EXPORT
/*
 * Under Borland and Visual C++, use __declspec
 */
#if defined(__BORLANDC__) || defined(_MSC_VER)
    #define RI_EXPORT __declspec(dllexport)
    #define RI_IMPORT __declspec(dllimport)
/*
 * Otherwise, don't use
 */
#else
    #define RI_EXPORT
    #define RI_IMPORT
#endif


/*
 * Data to the display drivers will be provided in a variety of formats
 * and byte orders. The display driver can then choose which formats and
 * byte orders it would prefer that data in.
 */
#define PkDspyNone		0
#define PkDspyFloat32		1
#define PkDspyUnsigned32	2
#define PkDspySigned32		3
#define PkDspyUnsigned16	4
#define PkDspySigned16		5
#define PkDspyUnsigned8		6
#define PkDspySigned8		7
/* Types for optional parameters */
#define PkDspyString		8
#define PkDspyMatrix		9

/*
 * Byte order can be specified by "or"ing in one of two values.
 */
#define PkDspyByteOrderHiLo	0x2000
#define PkDspyByteOrderLoHi	0x4000

/*
 * If byte orders aren't specified, the machine default is assumed.
 */
#if defined(__mips)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__mips64)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__sparc)
#define PkDspyByteOrderNative	PkDspyByteOrderHiLo
#elif defined(__alpha)
#define PkDspyByteOrderNative	PkDspyByteOrderLoHi
#else
#define PkDspyByteOrderNative	PkDspyByteOrderLoHi
#endif

/*
 * Basic data types
 */
typedef char PtDspySigned8;
typedef unsigned char PtDspyUnsigned8;
typedef short PtDspySigned16;
typedef unsigned short PtDspyUnsigned16;
#if defined(__mips64) || defined(__alpha)
typedef int PtDspySigned32;
typedef unsigned int PtDspyUnsigned32;
#else
typedef long PtDspySigned32;
typedef unsigned long PtDspyUnsigned32;
#endif
typedef float PtDspyFloat32;

/*
 * All functions return one of the following enumerated error codes:
 */
typedef enum {
    PkDspyErrorNone = 0,	/* successful completion */
    PkDspyErrorNoMemory,	/* unable to allocate memory */
    PkDspyErrorUnsupported,	/* unsupported operation requested */
    PkDspyErrorBadParams,	/* bad parameters */
    PkDspyErrorNoResource,	/* no resource available, file not found... */
    PkDspyErrorUndefined	/* no other error messages appropriate */
} PtDspyError;

typedef enum {
    PkSizeQuery,
    PkOverwriteQuery,
    PkNextDataQuery
} PtDspyQueryType;

typedef void *PtDspyImageHandle;	/* Opaque type */

typedef struct uparam {
    RtToken name;
    char vtype, vcount;
#ifdef __BORLANDC__
    char pad1, pad2;		/* Waste space for Visual C++ compatibility */
#endif
    RtPointer value;
    int nbytes;
} UserParameter;

typedef struct {
    char *name;
    unsigned type;
} PtDspyDevFormat;

#define PkDspyFlagsWantsScanLineOrder		0x01
#define PkDspyFlagsWantsEmptyBuckets		0x02
#define PkDspyFlagsWantsNullEmptyBuckets	0x04
typedef struct {
    int flags;
} PtFlagStuff;

typedef struct {
    PtDspyUnsigned8 overwrite;
    PtDspyUnsigned8 interactive;
} PtDspyOverwriteInfo;

typedef struct {
    PtDspyUnsigned32 width;
    PtDspyUnsigned32 height;
    PtDspyFloat32 aspectRatio;
} PtDspySizeInfo;

/*
 * Display drivers are written as shared objects with 4 required functions:
 *
 * DspyImageOpen 
 *     initialize the display driver, allocate necessary space, check image
 *     size, and specify the format in which incoming data will arrive. 
 * DspyImageQuery 
 *     query the display driver for image size (if not specified in the open
 *     call) and aspect ratio. 
 * DspyImageData 
 *     send data to the display driver. 
 * DspyImageClose 
 *     let the display driver free resources. 
 * And, optionally: 
 *
 * DspyImageDelayClose 
 *     let the display driver handle user interaction or anything else that
 *     might delay the shutdown of the display driver. 
 */
#ifdef __cplusplus
extern "C" {
#endif

extern PtDspyError RI_EXPORT
    DspyImageOpen(PtDspyImageHandle *image,
        const char *drivername, const char *filename, int width, int height,
        int paramCount, const UserParameter *parameters, int iFormatCount,
        PtDspyDevFormat *format, PtFlagStuff *flagstuff),
    DspyImageQuery(PtDspyImageHandle pvImage, PtDspyQueryType type,
        int size, void *p),
    DspyImageClose(PtDspyImageHandle pvImage),
    DspyImageData(PtDspyImageHandle image, int xmin, int xmax_plusone,
        int ymin, int ymax_plusone, int entrysize, const unsigned char *data),
    DspyImageDelayClose(PtDspyImageHandle);

#ifdef __cplusplus
}
#endif

#include "dspyhlpr.h"		/* Helpers for writing display drivers */

#endif
