//
// sl.h - Classes and built-in functions used by RenderDotC C++ shaders.
//
// RenderDotC (R) is:
//      Copyright (c) 1996-2001 by Dot C Software, Inc.
//      All Rights Reserved.
// RenderDotC (R) is a registered trademark of Dot C Software, Inc.
//

#ifndef SL_H
#define SL_H
#if defined(__sgi) || (_MSC_VER >= 1000)
#pragma once
#endif
#ifdef _MSC_VER
#pragma warning(disable: 4101 4190)	// Disable 'unreferenced local variable'
#endif

#include <errno.h>			// For errno
#include <math.h>			// For sin
#include <float.h>			// For FLT_EPSILON
#include <stddef.h>			// For offsetof
#include <stdarg.h>			// For va_list
#ifdef __BORLANDC__
#include <mem.h>			// For memset
#else
#include <string.h>			// For memset
#endif
#include "ridefs.h"			// For RtInt
#undef SO_TYPE
#include "evaldefs.h"			// For SO_TYPE

const float PI   = 3.14159265358979323846F;
const float PI_2 = 1.57079632679489661923F;
const float PI_4 = 0.78539816339744830962F;

inline RtBoolean epsilon(float a)
    { return a < FLT_EPSILON && a > -FLT_EPSILON; }

#undef RI_IMPORT
#undef RI_EXPORT
//
// Under Borland and Visual C++, use __declspec
//
#if defined(__BORLANDC__) || defined(_MSC_VER)
    #define RI_EXPORT __declspec(dllexport)
    #define RI_IMPORT __declspec(dllimport)
//
// Otherwise, don't use
//
#else
    #define RI_EXPORT
    #define RI_IMPORT
#endif

//
// SL types
//
class Point {
public:
#ifdef _HPUX_SOURCE
    Point();
#else
    Point() {}
#endif
    Point(RtFloat c) { x = y = z = c; }
    Point(RtFloat xx, RtFloat yy, RtFloat zz) { x = xx; y = yy; z = zz; }
    Point(const Point &p) { x = p.x; y = p.y; z = p.z; }

    Point operator-() const { return Point(-x, -y, -z); }
    Point &operator=(RtFloat c)
	{ x = y = z = c; return *this; }
    Point &operator=(const Point &p)
	{ x = p.x; y = p.y; z = p.z; return *this; }
    Point &operator+=(RtFloat c)
	{ x += c; y += c; z += c; return *this; }
    Point &operator+=(const Point &p)
	{ x += p.x; y += p.y; z += p.z; return *this; }
    Point &operator-=(RtFloat c)
	{ x -= c; y -= c; z -= c; return *this; }
    Point &operator-=(const Point &p)
	{ x -= p.x; y -= p.y; z -= p.z; return *this; }
    Point &operator*=(RtFloat c)
	{ x *= c; y *= c; z *= c; return *this; }
    Point &operator*=(const Point &p)
	{ x *= p.x; y *= p.y; z *= p.z; return *this; }
    Point &operator^=(const Point &p)
	{ return *this = *this ^ p; }
    Point &operator/=(RtFloat d)
	{ if (!epsilon(d)) { d = 1.0F / d; x *= d; y *= d; z *= d; }
	  return *this; }
    Point &operator/=(const Point &p)
	{ if (!epsilon(p.x) && !epsilon(p.y) && !epsilon(p.z))
	  { x /= p.x; y /= p.y; z /= p.z; } return *this; }

    friend Point operator+(const Point &p1, const Point &p2)
	{ return Point(p1.x + p2.x, p1.y + p2.y, p1.z + p2.z); }
    friend Point operator-(const Point &p1, const Point &p2)
	{ return Point(p1.x - p2.x, p1.y - p2.y, p1.z - p2.z); }
    friend Point operator*(RtFloat f, const Point &p2)
	{ return Point(f * p2.x, f * p2.y, f * p2.z); }
    friend Point operator*(const Point &p1, RtFloat f)
	{ return Point(p1.x * f, p1.y * f, p1.z * f); }
    friend Point operator*(const Point &p1, const Point &p2)
	{ return Point(p1.x * p2.x, p1.y * p2.y, p1.z * p2.z); }
    friend RtFloat dot(const Point &p1, const Point &p2)
	{ return p1.x * p2.x + p1.y * p2.y + p1.z * p2.z; }
    friend Point operator^(const Point &p1, const Point &p2)
	{ return Point(p1.y * p2.z - p1.z * p2.y,
		       p1.z * p2.x - p1.x * p2.z,
		       p1.x * p2.y - p1.y * p2.x); }
    friend Point operator/(const Point &p1, RtFloat d)
	{ if (epsilon(d)) return p1;
	  d = 1.0F / d; return Point(p1.x * d, p1.y * d, p1.z * d); }
    friend Point operator/(const Point &p1, const Point &p2)
	{ if (epsilon(p2.x) || epsilon(p2.y) || epsilon(p2.z)) return p1;
	  return Point(p1.x / p2.x, p1.y / p2.y, p1.z / p2.z); }
    friend RtBoolean operator==(const Point &p1, const Point &p2)
	{ return (p1.x == p2.x && p1.y == p2.y && p1.z == p2.z); }
    friend RtBoolean operator!=(const Point &p1, const Point &p2)
	{ return (p1.x != p2.x || p1.y != p2.y || p1.z != p2.z); }

    RtFloat x, y, z;
};

class Color {
public:
#ifdef _HPUX_SOURCE
    Color();
#else
    Color() {}
#endif
    Color(RtFloat c) { r = g = b = c; }
    Color(RtFloat rr, RtFloat gg, RtFloat bb) { r = rr; g = gg; b = bb; }
    Color(const Color &c) { r = c.r; g = c.g; b = c.b; }
    Color(const Point &p) { r = p.x; g = p.y; b = p.z; }

    Color &operator=(const Color &c)
	{ r = c.r; g = c.g; b = c.b; return *this; }
    Color &operator+=(const Color &c)
	{ r += c.r; g += c.g; b += c.b; return *this; }
    Color &operator-=(const Color &c)
	{ r -= c.r; g -= c.g; b -= c.b; return *this; }
    Color &operator*=(RtFloat m)
	{ r *= m; g *= m; b *= m; return *this; }
    Color &operator*=(const Color &c)
	{ r *= c.r; g *= c.g; b *= c.b; return *this; }
    Color &operator/=(RtFloat m)
	{ if (!epsilon(m)) { m = 1.0F / m; r *= m; g *= m; b *= m; }
	  return *this; }
    Color &operator/=(const Color &c)
	{ if (!epsilon(c.r) && !epsilon(c.g) && !epsilon(c.b))
	  { r /= c.r; g /= c.g; b /= c.b; } return *this; }
    friend Color operator+(const Color &c1, const Color &c2)
	{ return Color(c1.r + c2.r, c1.g + c2.g, c1.b + c2.b); }
    friend Color operator-(const Color &c1, const Color &c2)
	{ return Color(c1.r - c2.r, c1.g - c2.g, c1.b - c2.b); }
    friend Color operator*(RtFloat f, const Color &c2)
	{ return Color(f * c2.r, f * c2.g, f * c2.b); }
    friend Color operator*(const Color &c1, RtFloat f)
	{ return Color(c1.r * f, c1.g * f, c1.b * f); }
    friend Color operator*(const Color &c1, const Color &c2)
	{ return Color(c1.r * c2.r, c1.g * c2.g, c1.b * c2.b); }
    friend RtFloat dot(const Color &c1, const Color &c2)
	{ return c1.r * c2.r + c1.g * c2.g + c1.b * c2.b; }
    friend Color operator/(const Color &c1, RtFloat d)
	{ if (epsilon(d)) return c1;
	  d = 1.0F / d; return Color(c1.r * d, c1.g * d, c1.b * d); }
    friend Color operator/(const Color &c1, const Color &c2)
	{ if (epsilon(c2.r) || epsilon(c2.g) || epsilon(c2.b)) return c1;
	  return Color(c1.r / c2.r, c1.g / c2.g, c1.b / c2.b); }
    friend RtBoolean operator==(const Color &c1, const Color &c2)
	{ return (c1.r == c2.r && c1.g == c2.g && c1.b == c2.b); }
    friend RtBoolean operator!=(const Color &c1, const Color &c2)
	{ return (c1.r != c2.r || c1.g != c2.g || c1.b != c2.b); }

    RtFloat r, g, b;			// Red, green and blue model
};

extern "C" {
Color RI_IMPORT hsv(const Color &c);		// From hsv to rgb
Color RI_IMPORT hsl(const Color &c);
Color RI_IMPORT XYZ(const Color &c);
Color RI_IMPORT xyY(const Color &c);
Color RI_IMPORT YIQ(const Color &c);
}

class Matrix {
public:
    Matrix() {}

    Matrix &operator=(const Matrix &m)
	{ if (this != &m) memcpy(a, m.a, 16 * sizeof(RtFloat)); return *this; }
    Matrix &operator=(RtFloat d)
	{ memset(a, 0, 16 * sizeof(RtFloat));
	  a[0][0] = a[1][1] = a[2][2] = a[3][3] = d; return *this; }

    RtFloat a[4][4];
};

extern "C" {
Matrix RI_IMPORT projectss(RtToken fromstring, RtToken tostring);
Matrix RI_IMPORT projectms(const Matrix &frommatrix, RtToken tostring);
Matrix RI_IMPORT projectsm(RtToken fromstring, const Matrix &tomatrix);
Matrix RI_IMPORT projectmm(const Matrix &frommatrix, const Matrix &tomatrix);
Point RI_IMPORT transform(const Matrix &m, const Point &p);
Point RI_IMPORT vtransform(const Matrix &m, const Point &p);
Point RI_IMPORT ntransform(const Matrix &m, const Point &p);
}

//
// Mathematical functions
//
#ifdef __BORLANDC__
#define acosf acos
#define asinf asin
#define atan2f atan2
#define atanf atan
#define ceilf ceil
#define cosf cos
#define expf exp
#define fabsf fabs
#define floorf floor
#define fmodf fmod
#define logf log
#define powf pow
#define sinf sin
#define sqrtf sqrt
#define tanf tan
#endif

extern "C" {
typedef RtFloat (*RtNoise1Func)(RtFloat);
typedef RtFloat (*RtNoise2Func)(RtFloat, RtFloat);
typedef RtFloat (*RtNoise3Func)(const Point &);
typedef RtFloat (*RtNoise4Func)(const Point &, RtFloat);
extern RtNoise1Func RI_IMPORT fnoisef;
extern RtNoise2Func RI_IMPORT fnoiseff;
extern RtNoise3Func RI_IMPORT fnoisep;
extern RtNoise4Func RI_IMPORT fnoisepf;
}

inline float radians(float degrees) { return PI * degrees / 180; }
inline float degrees(float radians) { return 180 * radians / PI; }
inline float atanf(float y, float x) { return atan2f(y, x); }
inline float log(float x, float base) { return logf(x) / logf(base); }
inline float fpow(float x, float y)
    { return x < 0 && fmodf(y, 1.0) != 0 ? 0 : powf(x, y); }
inline float inversesqrt(float x) { return x > 0 ? 1.0f / sqrtf(x) : 0.0f; }
inline float mod(float a, float b)
    { return a < 0 ? fabsf(b) + fmodf(a, b) : fmodf(a, b); }
inline float sign(float x) { return x < 0 ? -1.0f : x > 0 ? 1.0f : 0.0f; }
inline RtInt clamp(RtInt a, RtInt min, RtInt max)
    { return a < min ? min : a > max ? max : a; }
inline float clamp(float a, float min, float max)
    { return a < min ? min : a > max ? max : a; }
inline float round(float x)
    { return x < 0 ? ceilf(x - 0.5f) : floorf(x + 0.5f); }
inline float mix(RtFloat low, RtFloat high, RtFloat value)
    { return low + (high - low) * value; }
inline float step(float min, float value) { return value < min ? 0.0f : 1.0f; }

extern "C" {
float RI_IMPORT fdiv(float a, float b);
float RI_IMPORT fminv(RtInt n, ...);
Color RI_IMPORT cminv(RtInt n, ...);
Point RI_IMPORT pminv(RtInt n, ...);
float RI_IMPORT fmaxv(RtInt n, ...);
Color RI_IMPORT cmaxv(RtInt n, ...);
Point RI_IMPORT pmaxv(RtInt n, ...);
Color RI_IMPORT cclamp(const Color &f, const Color &min, const Color &max);
Point RI_IMPORT pclamp(const Point &f, const Point &min, const Point &max);
float RI_IMPORT smoothstep(float min, float max, float value);
float RI_IMPORT fsplinev(const char *basis, float value, RtInt n, ...);
Color RI_IMPORT csplinev(const char *basis, float value, RtInt n, ...);
Point RI_IMPORT psplinev(const char *basis, float value, RtInt n, ...);
RtFloat RI_IMPORT filterstep(RtFloat edge, RtFloat val, RtFloat w, ...);
float RI_IMPORT frandom();
Color RI_IMPORT crandom();
Point RI_IMPORT prandom();
RtFloat RI_IMPORT fperlinf(RtFloat v);
RtFloat RI_IMPORT fperlinff(RtFloat u, RtFloat v);
RtFloat RI_IMPORT fperlinp(const Point &pt);
RtFloat RI_IMPORT fperlinpf(const Point &p, RtFloat w);
RtFloat RI_IMPORT flewisf(RtFloat v);
RtFloat RI_IMPORT flewisff(RtFloat u, RtFloat v);
RtFloat RI_IMPORT flewisp(const Point &pt);
RtFloat RI_IMPORT flewispf(const Point &p, RtFloat w);
Color RI_IMPORT cnoisef(RtFloat v);
Color RI_IMPORT cnoiseff(RtFloat u, RtFloat v);
Color RI_IMPORT cnoisep(const Point &pt);
Color RI_IMPORT cnoisepf(const Point &pt, RtFloat w);
Point RI_IMPORT pnoisef(RtFloat v);
Point RI_IMPORT pnoiseff(RtFloat u, RtFloat v);
Point RI_IMPORT pnoisep(const Point &pt);
Point RI_IMPORT pnoisepf(const Point &pt, RtFloat w);
RtFloat RI_IMPORT fpernoisef(RtFloat x, RtFloat period);
RtFloat RI_IMPORT fpernoiseff(RtFloat x, RtFloat y,
    RtFloat xperiod, RtFloat yperiod);
RtFloat RI_IMPORT fpernoisep(const Point &p, const Point &period);
RtFloat RI_IMPORT fpernoisepf(const Point &p, RtFloat w, const Point &pperiod,
    RtFloat wperiod);
Color RI_IMPORT cpernoisef(RtFloat v, RtFloat period);
Color RI_IMPORT cpernoiseff(RtFloat u, RtFloat v, RtFloat uperiod,
    RtFloat vperiod);
Color RI_IMPORT cpernoisep(const Point &pt, const Point &period);
Color RI_IMPORT cpernoisepf(const Point &pt, RtFloat w, const Point &pperiod,
    RtFloat wperiod);
Point RI_IMPORT ppernoisef(RtFloat v, RtFloat period);
Point RI_IMPORT ppernoiseff(RtFloat u, RtFloat v, RtFloat uperiod,
    RtFloat vperiod);
Point RI_IMPORT ppernoisep(const Point &pt, const Point &period);
Point RI_IMPORT ppernoisepf(const Point &pt, RtFloat w, const Point &pperiod,
    RtFloat wperiod);
RtFloat RI_IMPORT fcellnoisef(RtFloat x);
RtFloat RI_IMPORT fcellnoiseff(RtFloat x, RtFloat y);
RtFloat RI_IMPORT fcellnoisep(const Point &p);
RtFloat RI_IMPORT fcellnoisepf(const Point &p, RtFloat w);
Point RI_IMPORT pcellnoisef(RtFloat v);
Point RI_IMPORT pcellnoiseff(RtFloat u, RtFloat v);
Point RI_IMPORT pcellnoisep(const Point &pt);
Point RI_IMPORT pcellnoisepf(const Point &pt, RtFloat w);
Color RI_IMPORT ccellnoisef(RtFloat v);
Color RI_IMPORT ccellnoiseff(RtFloat u, RtFloat v);
Color RI_IMPORT ccellnoisep(const Point &pt);
Color RI_IMPORT ccellnoisepf(const Point &pt, RtFloat w);
}

//
// String functions
//
extern "C" {
char RI_IMPORT *concatv(RtInt n, ...);
char RI_IMPORT *format(const char *pattern, ...);
RtBoolean RI_IMPORT match(const char *pattern, const char *subject);
}


//
// Geometric functions
//
inline RtFloat xcomp(const Point &p) { return p.x; }
inline RtFloat ycomp(const Point &p) { return p.y; }
inline RtFloat zcomp(const Point &p) { return p.z; }
inline void setxcomp(Point &p, RtFloat x) { p.x = x; }
inline void setycomp(Point &p, RtFloat y) { p.y = y; }
inline void setzcomp(Point &p, RtFloat z) { p.z = z; }

extern "C" {
RtFloat RI_IMPORT compp(const Point &p, RtFloat index);
void RI_IMPORT setcompp(Point &p, RtFloat index, RtFloat value);
Point RI_IMPORT pmix(const Point &p0, const Point &p1, RtFloat value);
RtFloat RI_IMPORT length(const Point &p);
RtFloat RI_IMPORT distance(const Point &p1, const Point &p2);
RtFloat RI_IMPORT ptlined(const Point &p0, const Point &p1, const Point &q);
Point RI_IMPORT normalize(const Point &p);
Point RI_IMPORT faceforward(const Point &n, const Point &i, const Point &g);
Point RI_IMPORT reflect(const Point &i, const Point &n);
Point RI_IMPORT refract(const Point &i, const Point &n, RtFloat eta);
Point RI_IMPORT rotate(const Point &q, RtFloat angle, const Point &p0,
    const Point &p1);
void RI_IMPORT fresnel7(const Point &i, const Point &n, RtFloat eta,
    RtFloat &Kr, RtFloat &Kt, Point &r, Point &t);
Point RI_IMPORT transforms(const char *tospace, const Point &p);
Point RI_IMPORT transformm(const Matrix &m, const Point &p);
Point RI_IMPORT transformss(const char *fromspace, const char *tospace,
    const Point &p);
Point RI_IMPORT transformsm(const char *fromspace, const Matrix &m,
    const Point &v);
Point RI_IMPORT vtransforms(const char *tospace, const Point &v);
Point RI_IMPORT vtransformm(const Matrix &m, const Point &v);
Point RI_IMPORT vtransformss(const char *fromspace, const char *tospace,
    const Point &v);
Point RI_IMPORT vtransformsm(const char *fromspace, const Matrix &m,
    const Point &v);
Point RI_IMPORT ntransforms(const char *tospace, const Point &n);
Point RI_IMPORT ntransformm(const Matrix &m, const Point &n);
Point RI_IMPORT ntransformss(const char *fromspace, const char *tospace,
    const Point &n);
Point RI_IMPORT ntransformsm(const char *fromspace, const Matrix &m,
    const Point &n);
RtFloat RI_IMPORT depth(const Point &p);
}

inline void fresnel5(const Point &i, const Point &n, RtFloat eta, RtFloat &Kr,
    RtFloat &Kt) { Point r, t; fresnel7(i, n, eta, Kr, Kt, r, t); }
inline void fresnel6(const Point &i, const Point &n, RtFloat eta, RtFloat &Kr,
    RtFloat &Kt, Point &r) { Point t; fresnel7(i, n, eta, Kr, Kt, r, t); }

//
// Color functions
//
extern "C" {
RtFloat RI_IMPORT compc(const Color &c, RtFloat index);
void RI_IMPORT setcompc(Color &c, RtFloat index, RtFloat value);
Color RI_IMPORT cmix(const Color &color1, const Color &color2, RtFloat value);
Color RI_IMPORT ctransforms(const char *tospace, const Color &c);
Color RI_IMPORT ctransformss(const char *fromspace, const char *tospace,
    const Color &c);
}

//
// Shading and lighting functions
//
extern "C" {
typedef Color (*RtBrdfFunc)(const Point &, const Point &, const Point &,
    RtFloat);
extern RtBrdfFunc RI_IMPORT specularbrdf;
}

struct SLlight {
    SLlight *next;
    Point L;				// Ray direction
    Color Cl;				// Ray color
    Color Ol;				// Ray opacity
    void *id;				// Unique id for message passing
};

extern "C" {
SLlight RI_IMPORT *newlight();
void RI_IMPORT freelight(SLlight *p);
Color RI_IMPORT diffuse(const SLlight *lights, const Point &N);
Color RI_IMPORT specular(const SLlight *lights, const Point &N,
    const Point &V, RtFloat roughness);
Color RI_IMPORT phong(const SLlight *lights, const Point &N, const Point &V,
    RtFloat size);
RtBoolean RI_IMPORT illuminates(const Point &position, const Point &axis,
    RtFloat angle, const Point &Ps);
RtBoolean RI_IMPORT illuminance(const Point &axis, RtFloat angle, const Point &L);
Color RI_IMPORT standardbrdf(const Point &L, const Point &N, const Point &V,
    RtFloat roughness);
Color RI_IMPORT bmrtbrdf(const Point &L, const Point &N, const Point &V,
    RtFloat roughness);
Color RI_IMPORT prmanbrdf(const Point &L, const Point &N, const Point &V,
    RtFloat roughness);
Color RI_IMPORT rdcbrdf(const Point &L, const Point &N, const Point &V,
    RtFloat roughness);
Color RI_IMPORT torrancebrdf(const Point &L, const Point &N, const Point &V,
    RtFloat roughness);
Color RI_IMPORT trace(const Point &, const Point &);
}

//
// Texture mapping functions
//
extern "C" {
RtFloat RI_IMPORT ftexture(const char *name, RtInt channel,
    RtFloat s0, RtFloat t0, RtFloat s1, RtFloat t1,
    RtFloat s2, RtFloat t2, RtFloat s3, RtFloat t3, ...);
Color RI_IMPORT ctexture(const char *name, RtInt channel,
    RtFloat s0, RtFloat t0, RtFloat s1, RtFloat t1,
    RtFloat s2, RtFloat t2, RtFloat s3, RtFloat t3, ...);
RtFloat RI_IMPORT fenvironment(const char *name, RtInt channel,
    const Point &d0, const Point &d1, const Point &d2, const Point &d3,
    const char *token1, ...);
Color RI_IMPORT cenvironment(const char *name, RtInt channel,
    const Point &d0, const Point &d1, const Point &d2, const Point &d3,
    const char *token1, ...);
RtFloat RI_IMPORT shadow(const char *name, RtInt channel,
    const Point &d0, const Point &d1, const Point &d2, const Point &d3,
    const char *token1, ...);
}
inline Point bump(const char *) { return 0.0; }

//
// Volume variable access functions
//
inline RtFloat incident(const char *, RtFloat ) { return 0; }
inline RtFloat incident(const char *, const Color &) { return 0; }
inline RtFloat incident(const char *, const Point &) { return 0; }
inline RtFloat opposite(const char *, RtFloat) { return 0; }
inline RtFloat opposite(const char *, const Color &) { return 0; }
inline RtFloat opposite(const char *, const Point &) { return 0; }

//
// Print function
//
extern "C" void RI_IMPORT slprintf(const char *format, ...);

//
// Message passing
//
extern "C" {
RtFloat RI_IMPORT atmosphere(SO_DETAIL destdetail, SO_TYPE desttype,
    char *varname, int arraylen, RtPointer value, RtInt index);
RtFloat RI_IMPORT displacement(SO_DETAIL destdetail, SO_TYPE desttype,
    char *varname, int arraylen, RtPointer value, RtInt index);
RtFloat RI_IMPORT lightsource(SO_DETAIL destdetail, SO_TYPE desttype,
    char *varname, int arraylen, const void *srclight, RtPointer value,
    RtInt index);
RtFloat RI_IMPORT surface(SO_DETAIL destdetail, SO_TYPE desttype,
    char *varname, int arraylen, RtPointer value, RtInt index);
}

//
// SIMD runner control
//
struct Set;
extern "C" {
struct Set RI_IMPORT *set_create(int nmembers);
struct Set RI_IMPORT *set_copy(const struct Set *s);
struct Set RI_IMPORT *set_assign(Set *dest, const struct Set *s);
void RI_IMPORT set_destroy(struct Set *s);
void RI_IMPORT set_makeempty(struct Set *s);
void RI_IMPORT set_makefull(struct Set *s);
void RI_IMPORT set_insert(struct Set *s, int n);
void RI_IMPORT set_remove(struct Set *s, int n);
int RI_IMPORT set_ismember(const struct Set *s, int n);
int RI_IMPORT set_isempty(struct Set *s);
void RI_IMPORT set_intersection(struct Set *s, const struct Set *s2);
void RI_IMPORT set_union(struct Set *s, const struct Set *s2);
void RI_IMPORT set_not(struct Set *s);
}

struct RunFlag;
extern "C" {
struct RunFlag RI_IMPORT *runflag_create(int nprocs);
void RI_IMPORT runflag_destroy(struct RunFlag* rf);
int RI_IMPORT runflag_isset(const struct RunFlag *rf, int proc);
int RI_IMPORT runflag_anyset(const struct RunFlag *rf);
void RI_IMPORT runflag_push(struct RunFlag *rf);
void RI_IMPORT runflag_pop(struct RunFlag *rf);
void RI_IMPORT runflag_if(struct RunFlag *rf, struct Set *c);
void RI_IMPORT runflag_suspend(struct RunFlag *rf, int proc, int levels);
void RI_IMPORT runflag_else(struct RunFlag *rf);
}

//
// DSO shadeop evaluation
//
struct DsoShadeop;
extern "C" {
struct DsoShadeop RI_IMPORT *dsolookup(const char *filename,
    const char *slname, const char *cname, const char *prototype);
RtFloat RI_IMPORT fdso(struct DsoShadeop *h, const char *format, ...);
Point RI_IMPORT pdso(struct DsoShadeop *h, const char *format, ...);
Color RI_IMPORT cdso(struct DsoShadeop *h, const char *format, ...);
const char RI_IMPORT *sdso(struct DsoShadeop *h, const char *format, ...);
RtVoid RI_IMPORT vdso(struct DsoShadeop *h, const char *format, ...);
}

#endif

